#!/usr/bin/env perl
# Author: Ann Davis <andavis@suse.com>
# Based on work by: Vijay Tripathi <v-vijayt@microsoft.com>
use File::Basename;
use File::Copy;

#
# Declare subroutines.
#

sub update_bootloader_file;
sub GetUUID;
#
# Declare globals.
#
my $bootloaderfile = "/etc/sysconfig/bootloader";
my $kernel_ver = `uname -r`;
my $matchkernel = "vmlinuz-" . $kernel_ver;

#
# main entry point
#

backup_bootloader_file();
update_bootloader_file();

#
# Subroutine Backup bootloader file
#
sub backup_bootloader_file() {
        my $bootloaderfilebackup = "";
        my $i=0;

        $bootloaderfilebackup = "/etc/sysconfig/" . basename($bootloaderfile) . ".backup" . $i;

        while ( -e $bootloaderfilebackup ) {
                $i ++;
                $bootloaderfilebackup = "/etc/sysconfig/" . basename($bootloaderfile) . ".backup" . $i;
        }

        print "\tBacking up $bootloaderfile to $bootloaderfilebackup\n";
        copy($bootloaderfile, "$bootloaderfilebackup");
}


#
# Subroutine Update bootloader File
#
sub update_bootloader_file() {
	my $bootloaderfiletmp = $bootloaderfile . ".tmp123";
	my $modified = 0;

	if ( -e $bootloaderfiletmp) {
		die "\tError $bootloaderfiletmp exists. Please rename the file and rerun this.\n";
	}
	
	if (!open(BOOTLOADER_CONF_FILE_TMP, ">$bootloaderfiletmp")) {
		die "\tError opening $bootloaderfiletmp. Running as root?\n";
	}
	
	if (!open(BOOTLOADER_CONF_FILE, "<$bootloaderfile")) {
		close BOOTLOADER_CONF_FILE_TMP;
		unlink $bootloaderfiletmp;
		die "\tError opening $bootloaderfile. Running as root?\n";
	}
		
	while ($line =  <BOOTLOADER_CONF_FILE>) {
		if ($line =~ /DEFAULT_APPEND=/) {
			$tmp=$line;
			chomp $line;
			@linearray = split('=', $line, 2);
			$optsstring=$linearray[1];
			@optsarray = split(/  */, $optsstring);
			@optstoedit = ();
			# find all the strings to be edited
			foreach $curopt (@optsarray)
			{
				if(($curopt =~ /\/dev\//) || ($curopt =~ /LABEL/)){
					push(@optstoedit,$curopt);
				}
			}
			# edit each string
			foreach $curopttoedit (@optstoedit){
				@var = split(/=/,$curopttoedit, 2);
				$uuid=GetUUID($var[1]);
				if ($uuid == 0) {
				$line =~ s/\/dev\/hd/\/dev\/sd/gi;
				} else {
				$line =~ s/$var[1]/$uuid/gi;
				}
			}
			printf BOOTLOADER_CONF_FILE_TMP $line."\n";
			$modified = 1;
		} else {
			printf BOOTLOADER_CONF_FILE_TMP $line;
		}		
	}
	
	close BOOTLOADER_CONF_FILE;
	close BOOTLOADER_CONF_FILE_TMP;
		
	if ($modified == 1) {
		move($bootloaderfiletmp, $bootloaderfile);
	} else {
		unlink $bootloaderfiletmp;
	}
}

#Subroutine to get uuid
sub GetUUID {
	#BLKID COMMAND OUTPUT
	if ($_[0] =~ /LABEL/) {
		$uuidfstr=`blkid -t $_[0]`;
	}
	elsif ($_[0] =~ /\/dev\//) {
		$uuidfstr=`blkid $_[0]`;
	}
	else {
		print "Error:  Cannot get UUID from string.\n";
	}
	#SPLITTING O/P IN THE STRINGS
	@uuid_arr = split(/  */,$uuidfstr);
	$uuid = 0;
	#REMOVING "" FROM UUID STRING
	foreach(@uuid_arr){
		if($_ =~ /^U/){
			$uuid = $_;
			$uuid =~ s/"//gi;
		}
	}
	$uuid;
}
