#!/usr/bin/env perl
# Author: Vijay Tripathi <v-vijayt@microsoft.com>
use File::Basename;
use File::Copy;

#
# Declare subroutines.
#

sub update_grub_file;
sub GetUUID;
#
# Declare globals.
#
my $grubfile = "/boot/grub/menu.lst";
my $kernel_ver = `uname -r`;
my $matchkernel = "vmlinuz-" . $kernel_ver;

#
# main entry point
#
chomp $matchkernel;

backup_grub_file();
update_grub_file();

#
# Subroutine Backup grub file
#
sub backup_grub_file() {
        my $grubfilebackup = "";
        my $i=0;

        $grubfilebackup = "/boot/grub/" . basename($grubfile) . ".backup" . $i;

        while ( -e $grubfilebackup ) {
                $i ++;
                $grubfilebackup = "/boot/grub/" . basename($grubfile) . ".backup" . $i;
        }

        print "\tBacking up $grubfile to $grubfilebackup\n";
        copy($grubfile, "$grubfilebackup");
}


#
# Subroutine Update Grub File
#
sub update_grub_file() {
	my $grubfiletmp = $grubfile . ".tmp123";
	my $modified = 0;

	if ( -e $grubfiletmp) {
		die "\tError $grubfiletmp exists. Please rename the file and rerun this.\n";
	}
	
	if (!open(GRUB_CONF_FILE_TMP, ">$grubfiletmp")) {
		die "\tError opening $grubfiletmp. Running as root?\n";
	}
	
	if (!open(GRUB_CONF_FILE, "<$grubfile")) {
		close GRUB_CONF_FILE_TMP;
		unlink $grubfiletmp;
		die "\tError opening $grubfile. Running as root?\n";
	}
		
	#print "\tLooking for kernel match - $matchkernel.\n";
		
	while ($line =  <GRUB_CONF_FILE>) {
		if ($line =~ /$matchkernel/) {

			if (($line =~ /\/dev\/*/) || ($line =~ /LABEL*/)) {
				$tmp=$line;
				chomp $line;
				@disk = split(/  */,$line);
				@alldevs = ();
				# find all the strings to be edited
				foreach $curdisk (@disk)
				{
					if(($curdisk =~ /\/dev\//) || ($curdisk =~ /LABEL/)){
						push(@alldevs,$curdisk);
					}
				}
				# edit each string
				foreach $curdev (@alldevs){
					@var = split(/=/,$curdev, 2);
					$uuid=GetUUID($var[1]);
					if ($uuid == 0) {
					$line =~ s/\/dev\/hd/\/dev\/sd/gi;
					} else {
					$line =~ s/$var[1]/$uuid/gi;
					}
				}
				printf GRUB_CONF_FILE_TMP $line."\n";
				$modified = 1;
			}
		} else {
			printf GRUB_CONF_FILE_TMP $line;
		}		
	}
	
	close GRUB_CONF_FILE;
	close GRUB_CONF_FILE_TMP;
		
	if ($modified == 1) {
		move($grubfiletmp, $grubfile);
	} else {
		unlink $grubfiletmp;
	}
}

#Subroutine to get uuid
sub GetUUID {
	#BLKID COMMAND OUTPUT
	if ($_[0] =~ /LABEL/) {
		$uuidfstr=`blkid -t $_[0]`;
	}
	elsif ($_[0] =~ /\/dev\//) {
		$uuidfstr=`blkid $_[0]`;
	}
	else {
		print "Error:  Cannot get UUID from string.\n";
	}
	#SPLITTING O/P IN THE STRINGS
	@uuid_arr = split(/  */,$uuidfstr);
	$uuid = 0;
	#REMOVING "" FROM UUID STRING
	foreach(@uuid_arr){
		if($_ =~ /^U/){
			$uuid = $_;
			$uuid =~ s/"//gi;
		}
	}
	$uuid;
}
